import React, {Component, PropTypes} from 'react';
import ReactDOM from 'react-dom/server';
import serialize from 'serialize-javascript';
import Helmet from 'react-helmet';

/**
 * Wrapper component containing HTML metadata and boilerplate tags.
 * Used in server-side code only to wrap the string output of the
 * rendered route component.
 *
 * The only thing this component doesn't (and can't) include is the
 * HTML doctype declaration, which is added to the rendered output
 * by the server.js file.
 */
export default class Html extends Component {

  static propTypes = {
    component: PropTypes.node,
    chunkJs: PropTypes.string,
    chunkCss: PropTypes.string,
  };

  render() {
    const { component, chunkJs, chunkCss } = this.props;
    const content = component ? ReactDOM.renderToStaticMarkup(component) : '';
    const head = Helmet.rewind();

    return (
      <html lang="zh-CN">
        <head>
          {head.base.toComponent() }
          {head.title.toComponent() }
          {head.meta.toComponent() }
          {head.link.toComponent() }
          {head.script.toComponent() }

          <meta charSet="utf-8"/>
          <link rel="shortcut icon" href="/favicon.ico"/>
          <meta name="viewport" content="width=device-width,initial-scale=1, user-scalable=0,maximum-scale=1"/>
          <meta name="format-detection" content="telephone=no, email=no"/>
          <meta name="apple-mobile-web-app-capable" content="yes"/>
          <link rel="apple-touch-icon" href="/apple-touch-icon.png" />
          {/* styles (will be present only in production with webpack extract text plugin) */}
          <link href={chunkCss} media="screen, projection"
            rel="stylesheet" type="text/css" charSet="UTF-8"/>

          {/* (will be present only in development mode) */}
          {/* outputs a <style/> tag with all bootstrap styles + App.scss + it could be CurrentPage.scss. */}
          {/* can smoothen the initial style flash (flicker) on page load in development mode. */}
          {/* ideally one could also include here the style for the current page (Home.scss, About.scss, etc) */}
          { /* Object.keys(assets.styles).length === 0 ? <style dangerouslySetInnerHTML={{__html: require('../theme/bootstrap.config.js') + require('../containers/BetaApp/style.scss')._style}}/> : null */}
        </head>
        <body>
          <div id="root" dangerouslySetInnerHTML={{ __html: content }}/>
          {__DEVELOPMENT__ && <script src="/js/betalib1-dll.js"/> }
          {__DEVELOPMENT__ && <script src={chunkJs} charSet="UTF-8"/> }
        </body>
      </html>
    );
  }
}
